package com.ruoyi.demo.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.ruoyi.common.core.validate.QueryGroup;
import com.ruoyi.demo.entity.App;
import com.ruoyi.demo.entity.Commercial;
import com.ruoyi.demo.entity.bo.CallBackBean;
import com.ruoyi.demo.entity.vo.AppVo;
import com.ruoyi.demo.mapper.AppMapper;
import com.ruoyi.demo.mapper.CommercialMapper;
import com.ruoyi.demo.service.AppService;
import com.ruoyi.demo.utils.SendCallBackUtil;
import com.ruoyi.demo.utils.UuidUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.client.RestTemplate;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;

@Service
public class AppServiceImpl implements AppService {
    @Autowired
    AppMapper appMapper;

    @Autowired
    CommercialMapper commercialMapper;

    @Override
    public void addApp(AppVo appVo) {
        App app = new App(appVo);
        //生成appkey
        app.setAppkey(UuidUtil.getUUID());
        //生成密钥
        app.setAppSecret(UuidUtil.getUUID());
        //时间
        app.setCreate_time(LocalDateTime.now());
        appMapper.insert(app);
    }

    @Override
    public void deleteApp(String[] appKeys) {
        appMapper.deleteBatchIds(Arrays.asList(appKeys));
    }

    @Override
    public PageInfo<App> getAppListByCommercialId(AppVo appVo) {
        QueryWrapper<App> queryWrapper = new QueryWrapper<>();
        if (appVo.getSearchText() != null && !appVo.getSearchText().equals("")){
            queryWrapper.likeRight("app_key",appVo.getSearchText());
            queryWrapper.or().like("app_name",appVo.getSearchText());
            queryWrapper.or().likeRight("commercial_name",appVo.getSearchText());
        }

        PageHelper.startPage(appVo.getPageNum(),appVo.getPageSize());
        List<App> apps = appMapper.selectList(queryWrapper);
        PageInfo<App> pageInfo = new PageInfo<>(apps);
        return pageInfo;
    }

    @Override
    public App getApp(AppVo appVo) {
        App app = appMapper.selectById(appVo.getAppkey());
        return app;
    }

    @Override
    public void updateApp(AppVo appVo) {
        UpdateWrapper<App> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("appkey",appVo.getAppkey())
            .set("app_logo",appVo.getAppLogo())
            .set("app_name",appVo.getAppName())
            .set("app_introduce",appVo.getAppIntroduce())
            .set("commercial_id",appVo.getCommercialId())
            .set("commercial_name",appVo.getCommercialName())
            .set("state",appVo.getState())
            .set("call_back",appVo.getCallBack())
            .set("prove_1",appVo.getProve1())
            .set("prove_2",appVo.getProve2())
            .set("prove_3",appVo.getProve3())
            .set("prove_4",appVo.getProve4())
            .set("prove_5",appVo.getProve5())
            .set("annotation",appVo.getAnnotation());
        appMapper.update(null,updateWrapper);
    }

    @Override
    public void testCallBack(String callBack) {
        SendCallBackUtil.testCallBack(callBack);
    }

    @Override
    public Page<App> getAppList(AppVo appVo) {
        QueryWrapper<App> queryWrapper = new QueryWrapper<>();
        if (appVo.getSearchText() != null && !appVo.getSearchText().equals("")){
            queryWrapper.likeRight("app_key",appVo.getSearchText());
            queryWrapper.or().like("app_name",appVo.getSearchText());
            queryWrapper.or().likeRight("commercial_name",appVo.getSearchText());
        }

        Page<App> page = new Page<>();
        page.setCurrent(appVo.getPageNum());
        page.setSize(appVo.getPageSize());
        Page<App> appPage = appMapper.selectPage(page, queryWrapper);
        return appPage;
    }

    @Override
    public List<Commercial> getCommercialList() {
        QueryWrapper<Commercial> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("commercial_id","commercial_name");
        List<Commercial> commercials = commercialMapper.selectList(queryWrapper);
        return commercials;
    }
}
